/**
 * $Id
 * Copyright \251 2002 Sun Microsystems, Inc. All rights reserved.
 *
 * U.S. Government Rights - Commercial software. Government users are subject
 * to the Sun Microsystems, Inc. standard license agreement and applicable
 * provisions of the FAR and its supplements.
 *
 * Use is subject to license terms.
 *
 * This distribution may include materials developed by third parties.Sun, Sun
 * Microsystems, the Sun logo and Java are trademarks or registered trademarks
 * of Sun Microsystems, Inc. in the U.S. and other countries.
 *
 * Copyright \251 2002 Sun Microsystems, Inc. Tous droits r\351serv\351s.
 *
 * Droits du gouvernement am\351ricain, utlisateurs gouvernmentaux - logiciel
 * commercial. Les utilisateurs gouvernmentaux sont soumis au contrat de licence * standard de Sun Microsystems, Inc., ainsi qu aux dispositions en vigueur de
 * la FAR [ (Federal Acquisition Regulations) et des suppl\351ments \340
 * celles-ci.
 *
 * Distribu\351 par des licences qui en restreignent l'utilisation.
 *
 * Cette distribution peut comprendre des composants d\351velopp\351s pardes
 * tierces parties.Sun, Sun Microsystems, le logo Sun et Java sont des marques
 * de fabrique ou des marques d\351pos\351es de Sun Microsystems, Inc. aux
 * Etats-Unis et dans d'autres pays.
 * Other Netscape logos, product names, and service names are
 * also trademarks of Netscape Communications Corporation,
 * which may be registered in other countries.
 */

import com.sun.identity.liberty.ws.authnsvc.*;
import com.sun.identity.liberty.ws.disco.*;
import com.sun.identity.liberty.ws.security.*;
import com.sun.identity.liberty.ws.authnsvc.protocol.*;
import java.util.*;
import org.w3c.dom.*;

/**
 * This class <code>PlainClient</code> is a sample client for the 
 * liberty authentication  service. This sample uses a plain authentication
 * mechanism.
 */
public class PlainClient {

    public static void main( String[] args) {

        if(args.length < 3) {
           System.out.println("Usage: java PlainClient <url> <uid> <password>");
        }

        String url = args[0];
        String userID = args[1];
        String password = args[2]; 

	try {
            // create a sasl request
	    SASLRequest saslReq =
                        new SASLRequest(AuthnSvcConstants.MECHANISM_PLAIN);
            saslReq.setAuthzID(userID);
            saslReq.setAdvisoryAuthnID(password);

            System.out.println("Sending first sasl request");
            SASLResponse saslResp = AuthnSvcClient.sendRequest(saslReq, url);
            String statusCode = saslResp.getStatusCode();
            System.out.println("Response code = " + statusCode);
            if (!statusCode.equals(SASLResponse.CONTINUE)) {
		System.exit(0);
            }

            String serverMechanism = saslResp.getServerMechanism();
            System.out.println("Response serverMechanism = "+ serverMechanism);

            // Here, the data string depends on sasl response, but assume
            //  a plain authentication mechanism for this sample.

            saslReq  = new SASLRequest(serverMechanism);
            String dataStr = userID + "\0" + userID + "\0" + password;
            saslReq.setData(dataStr.getBytes("UTF-8"));
            saslReq.setRefToMessageID(saslResp.getMessageID());

            System.out.println("Sending second request");
            saslResp = AuthnSvcClient.sendRequest(saslReq, url);
            statusCode = saslResp.getStatusCode();
            System.out.println("Response code = " + statusCode);

            // Get the discovery resource offering from the sasl response.
            ResourceOffering ro = saslResp.getResourceOffering();
            System.out.println("ResourceOffering = " + ro);

            //Look for the credentials from the sasl response.
            List list = saslResp.getCredentials();
            SecurityAssertion assertion = null;

            if (list != null && !list.isEmpty()) {

		Element assertionE = (Element)list.get(0);
		assertion = new SecurityAssertion(assertionE);
		System.out.println("\n\nAssertion = " +
                                   assertion.toString(true,true));
		System.out.println("Assertion is bearer " +
                                   assertion.isBearer());
            }

            ResourceID roID = ro.getResourceID();
            RequestedService rs =
                    new RequestedService(null, "urn:liberty:id-sis-pp:2003-08");

            List rss = new ArrayList();
            rss.add(rs);
            Query query = new Query(roID, rss);

            String soapURI =
		((Description)ro.getServiceInstance().getDescription().get(0))
				.getEndpoint();

            DiscoveryClient discoClient = null;
            if(assertion != null) {
               discoClient = new DiscoveryClient(assertion, soapURI, null);
            } else {
               discoClient = new DiscoveryClient(soapURI, null);
            }

            System.out.println("Sending Query " + query);
            QueryResponse qResp = discoClient.getResourceOffering(query);

            System.out.println("\n\nQueryResponse = " + qResp);


	} catch (Throwable ex) {
            ex.printStackTrace();
	}
        System.exit(0);
    }

}

